require 'active_support'
require 'orm_adapter'
require 'active_support/core_ext/numeric/time'
require 'securerandom'

module Devision
  autoload :TokenGenerator, 'devision/token_generator'
  autoload :Errors, 'devision/errors'

  module Models
    autoload :Config, 'devision/models/config'
    autoload :Confirmable, 'devision/models/confirmable'
    autoload :Authenticatable, 'devision/models/authenticatable'
    autoload :DatabaseAuthenticatable, 'devision/models/database_authenticatable'
    autoload :Recoverable, 'devision/models/recoverable'
    autoload :Rememberable, 'devision/models/rememberable'
    autoload :Validatable, 'devision/models/validatable'
    autoload :Tokenizable, 'devision/models/tokenizable'
  end

  # Devision configuration settings (for initializers)
  require 'devision/configuration_options'

  # Generate a user friendly 'nice' string randomly to be used as token.
  def self.nice_token
    SecureRandom.urlsafe_base64(15).tr('lIO0', 'sxyz')
  end

  # constant-time comparison algorithm to prevent timing attacks
  def self.secure_compare(a, b)
    return false if a.blank? || b.blank? || a.bytesize != b.bytesize
    l = a.unpack "C#{a.bytesize}"

    res = 0
    b.each_byte { |byte| res |= byte ^ l.shift }
    res == 0
  end

  # Digests a password using bcrypt.
  def self.bcrypt(klass, password)
    ::BCrypt::Password.create("#{password}#{klass.pepper}", cost: klass.stretches).to_s
  end

  # Default way to setup Devision. Just call this in an initializer
  def self.setup
    yield(self)
    self.initialize!
  end

  protected
    def self.initialize!

      Devision.token_generator ||= begin
        if secret = Devision.secret_key
          Devision::TokenGenerator.new(
            Devision::CachingKeyGenerator.new(Devision::KeyGenerator.new(secret))
          )
        else
          raise Devision::Errors::Configuration.new('Devision.secret_key contain a value for encryption.')
        end
      end

    end

end
