module Devision
  module Models
    module Confirmable
      extend ActiveSupport::Concern

      # Fields required on the target Model
      def self.required_fields(klass)
        [:confirmation_token, :confirmed_at, :confirmation_sent_at]
      end

      included do

        attr_reader :raw_confirmation_token

        def confirm!
          pending_any_confirmation do
            if confirmation_period_expired?
              self.errors.add(:email, :confirmation_period_expired)
              return false
            end
            self.confirmation_token = nil
            self.confirmed_at = Time.now.utc
            save(validate: false)
          end
        end

        def confirmed?
          !!confirmed_at
        end

        def generate_confirmation_tokens
          raw, enc = Devision.token_generator.generate(self.class, :confirmation_token)
          @raw_confirmation_token = raw
          self.confirmation_token = enc
          self.confirmation_sent_at = Time.now.utc
        end

        def generate_confirmation_tokens!
          generate_confirmation_tokens && save(validate: false)
        end

        def clear_confirmation_token
          @raw_confirmation_token = nil
          self.confirmation_token = nil
          self.confirmation_sent_at = nil
        end

        # Checks whether the record requires any confirmation.
        def pending_any_confirmation
          if !confirmed?
            yield
          else
            self.errors.add(:email, :already_confirmed)
            false
          end
        end

        def confirmation_period_expired?
          self.class.confirm_within && (Time.now > self.confirmation_sent_at + self.class.confirm_within )
        end

        def confirmation_period_valid?
          self.class.allow_unconfirmed_access_for.nil? || (confirmation_sent_at && confirmation_sent_at.utc >= self.class.allow_unconfirmed_access_for.ago)
        end

      end

      module ClassMethods

        def confirm_by_token(raw_confirmation_token)
          original_token     = raw_confirmation_token
          saved_token = Devision.token_generator.digest(self, :confirmation_token, original_token)

          confirmable = find_or_initialize_with_error_by(:confirmation_token, saved_token)
          confirmable.confirm! if confirmable.persisted?
          confirmable.confirmation_token = original_token
          confirmable
        end

        Devision::Models::Config.add_options(self, :allow_unconfirmed_access_for, :confirmation_keys, :reconfirmable, :confirm_within)
      end


    end # Confirmable
  end # Models
end # Devision
