module Devision
  module Lockable
    extend ActiveSupport::Concern

    delegate :lock_strategy_enabled?, :unlock_strategy_enabled?, to: 'self.class'

    def self.required_fields(klass)
      attributes = []
      attributes << :failed_attempts if klass.lock_strategy_enabled?(:failed_attempts)
      attributes << :locked_at if klass.unlock_strategy_enabled?(:time)
      attributes << :unlock_token if klass.unlock_strategy_enabled?(:email)
      attributes
    end

    def lock_access!
      self.locked_at = Time.now.utc
      save(validate: false)
    end

    def unlock_access!
      self.locked_at = nil
      self.failed_attempts = 0 if respond_to?(:failed_attempts=)
      self.unlock_token = nil if respond_to?(:unlock_token=)
      save(validate: false)
    end

    def access_locked?
      !!locked_at && !lock_expired?
    end

    protected
      def attempts_exceeded?
        self.failed_attempts >= self.class.maximum_attempts
      end

      def last_attempt?
        self.failed_attempts == (self.class.maximum_attempts - 1)
      end

      def lock_expired?
        if unlock_strategy_enabled?(:time)
          locked_at && locked_at < self.class.unlock_in.ago
        else
          false
        end
      end

    module ClassMethods
      def unlock_access_by_token(unlock_token)
        original_token = unlock_token
        unlock_token = Devision.token_generator.digest(self, :unlock_token, unlock_token)
        
        lockable = find_or_initialize_with_error_by(:unlock_token, unlock_token)
        lockable.unlock_access! if lockable.persisted?
        lockable.unlock_token = original_token
        lockable
      end

      def unlock_strategy_enabled?(strategy)
        self.lock_strategy == strategy
      end

      Devision::Models::Config.add_options(self, :maximum_attempts, :lock_strategy, :unlock_strategy, :unlock_in, :unlock_keys)
    end

  end
end
